; HUFFMAN EXTRACTOR - by Sebastian Weber <webersebastian@yahoo.de>

#include "usgard.h"

.org 0
.db "Slideshow",0

	; First, initialize my pointers into the compressed data
	ld de, &Imagec
	ld hl, (&Imagec+1)
	add hl, de
	ld (&Huf_TreePtr), hl
	ld a,1
	ld (&CurPic),a

	; initialize my grayscale graphics
	#fncall gray_on
	ret c				; The carry will be set if to little memory, return to Usgard if that happens.

MainLoop:
	#fncall gray_off
	; load my pointers and show the pic nr (&CurPic)
	call CLEARLCD
	ld hl, &Imagec
	ld b,0
	ld a,(&CurPic)
	ld c,a
	add hl,bc
	adc hl,bc
	adc hl,bc
	ld c,(hl)
	inc hl
	ld b,(hl)
	inc hl
	ld a,(hl)
	ld hl, &Imagec
	add hl,bc
	ld (&Huf_DataPtr), hl
	ld (&Huf_DataBit), a
	di						; disable interrupts - theyre messing up my shadow registers
	#fncall gray_on
	Call &ExtractPic
	ei
WaitLoop:
	bit 7,a
	jr nz, EndProg
	or a
	jr nz, MainLoop
	call &CheckKeyboard
	jr WaitLoop

; checks the keyboard
; input: -
; output: a: 0 no action req, 1 action req, 255 exit req
; kills: -
CheckKeyboard:
	call OTH_ARROW
	xor 255
	ret z		; return if no key pressed
CheckKeyboard_GotKey:
	bit 6,a		; check for exit key
	jr z, CheckKeyboard_NoExit
	or 255		; Flag a program exit
	ret
CheckKeyboard_NoExit:
	and 10
	jr z, CheckKeyboard_NoPrev
	ld a,(&CurPic) ; Previous Picture
	dec a
	ret z
CheckKeyboard_PrevOK:
	ld (&CurPic),a
	ret
CheckKeyboard_NoPrev:
	ld a,(&CurPic)	; Next Picture
	inc a
	push hl
	ld hl,&Imagec 
	cp (hl)
	pop hl
	jr z, CheckKeyboard_NextOK
	jr c, CheckKeyboard_NextOK
	ld a,0
	ret
CheckKeyboard_NextOK:
	ld (&CurPic),a
	ret
	
EndProg:
	#fncall gray_off
	call OTH_EXIT		; Return to Usgard

; ExtractPic - extracts next pic located at (&Huf_DataPtr, &Huf_DataBit)
; IN:		-
; OUT:		a: Keyboard status (255 exit, 0 no kb, 1 kb)
; DESTROYS:
ExtractPic:
	ld hl, (PAGE1ADDR)
	ld de, (PAGE2ADDR)
	ld bc, 1024
	
ExtractPic_loop:
	Call &Huf_DecodeChar
	ld (hl),a
	Call &Huf_DecodeChar
	xor (hl)
	ld (de),a

	inc hl		; increment pointer to pic1
	inc de		; increment pointer to pic2
	dec bc		; decrement counter
	; check keyboard after each 2 scanlines (each 32 loopcycles)
	; if we have a keypress, chancel image decoding
	ld a,c
	and 31
	jr nz,SkipKBCheck
	call &CheckKeyboard
	or a
	ret nz
SkipKBCheck:
	ld a,0  ; check if counter is 0 - loop if not
	or c
	jr nz, ExtractPic_loop
	or b
	jr nz, ExtractPic_loop
	ret

; Huf_DecodeChar - decodes one character
; IN:		hl, b -> address
; OUT:		A -> decoded byte
; DESTROYS: HL', B'
Huf_DecodeChar:
	push hl					; Save registers
	push de
	push bc
	ld hl, (&Huf_DataPtr)	; Get current stream pointer
	ld a, (&Huf_DataBit)
	ld b,a
	exx
	ld hl, (&Huf_TreePtr)
	ld b, $80
Huf_DecodeChar_loop:
	call &Huf_GetBit		; Get the first node from the Tree
	jr nz, Huf_DecodeChar_gotleaf
	exx
	call &Huf_GetBit
	exx
	call nz, &Huf_SkipNode
	jr Huf_DecodeChar_loop

Huf_DecodeChar_gotleaf:		; Get the next 8 bit from the tree stream and return them
	ld a,b					; theire our data
	ld d,(hl)
	inc hl
	ld e,(hl)
Huf_Get8Bit_l1:
	add a,a
	jr z, Huf_Get8Bit_l2
	SLA e
	RL d
	jr Huf_Get8Bit_l1
Huf_Get8Bit_l2:
	ld a,d

	exx						; Store my stream pointer back
	ld (&Huf_DataPtr), hl
	ex af,af'
	ld a, b
	ld (&Huf_DataBit), a
	ex af,af'
	pop bc
	pop de
	pop hl
	ret

; Huf_SkipNode - skips a complete node w/ subnodes and and returns the adress of the following node or data
; IN:		HL->StreamPtr, B->BitNr
; OUT:		HL->new StreamPtr, B->new BitNr
; DESTROYS:	C,A
Huf_SkipNode:
	ld c, 0		; c is our left branching counter
Huf_SkipNode_gotnode
	inc c
Huf_SkipNode_loop:
	call &Huf_GetBit
	jr z, Huf_SkipNode_gotnode
	inc hl		; got a leaf, skip one byte
	dec c
	jr nz, Huf_SkipNode_loop
	ret

; Huf_GetBit - get the next bit from a Stream
; IN:	HL->StreamPtr, B->BitNr
; OUT:	HL->new StreamPtr, B->new BitNr, Z<>0 if bit set, Z=0 if bit unset
; DESTROYS: A
Huf_GetBit:
	ld a,(HL)
	and b
	RRC b
	jr nc, Huf_GetBit_l1
	inc hl
Huf_GetBit_l1:
	or a
	ret

Huf_DataPtr:
	.db 0,0
Huf_DataBit:
	.db 0
Huf_TreePtr:
	.db 0,0
CurPic:
	.db 0
Imagec:
	#incbin tempoc
.end
